import json

LIQEE_CONTROLLER_ABI = json.loads('''
[
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "paused",
        "type": "bool"
      }
    ],
    "name": "BorrowPaused",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "BorrowedAdded",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "BorrowedRemoved",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "collateralFactor",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "borrowFactor",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "supplyCapacity",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "borrowCapacity",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "distributionFactor",
        "type": "uint256"
      }
    ],
    "name": "MarketAdded",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "MarketEntered",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "MarketExited",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "paused",
        "type": "bool"
      }
    ],
    "name": "MintPaused",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldBorrowCapacity",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newBorrowCapacity",
        "type": "uint256"
      }
    ],
    "name": "NewBorrowCapacity",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldBorrowFactorMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newBorrowFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewBorrowFactor",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldCloseFactorMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newCloseFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewCloseFactor",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldCollateralFactorMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newCollateralFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewCollateralFactor",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldLiquidationIncentiveMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newLiquidationIncentiveMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewLiquidationIncentive",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "previousOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "NewOwner",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldPauseGuardian",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newPauseGuardian",
        "type": "address"
      }
    ],
    "name": "NewPauseGuardian",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "oldPendingOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newPendingOwner",
        "type": "address"
      }
    ],
    "name": "NewPendingOwner",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldPriceOracle",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newPriceOracle",
        "type": "address"
      }
    ],
    "name": "NewPriceOracle",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldRewardDistributor",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "_newRewardDistributor",
        "type": "address"
      }
    ],
    "name": "NewRewardDistributor",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldSupplyCapacity",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newSupplyCapacity",
        "type": "uint256"
      }
    ],
    "name": "NewSupplyCapacity",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "iToken",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "paused",
        "type": "bool"
      }
    ],
    "name": "RedeemPaused",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "bool",
        "name": "paused",
        "type": "bool"
      }
    ],
    "name": "SeizePaused",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "bool",
        "name": "paused",
        "type": "bool"
      }
    ],
    "name": "TransferPaused",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "_acceptOwner",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_collateralFactor",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_borrowFactor",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_supplyCapacity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_borrowCapacity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_distributionFactor",
        "type": "uint256"
      }
    ],
    "name": "_addMarket",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setAllBorrowPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setAllMintPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setAllRedeemPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_newBorrowCapacity",
        "type": "uint256"
      }
    ],
    "name": "_setBorrowCapacity",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_newBorrowFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setBorrowFactor",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setBorrowPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_newCloseFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setCloseFactor",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_newCollateralFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setCollateralFactor",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_newLiquidationIncentiveMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setLiquidationIncentive",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setMintPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_newPauseGuardian",
        "type": "address"
      }
    ],
    "name": "_setPauseGuardian",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address payable",
        "name": "newPendingOwner",
        "type": "address"
      }
    ],
    "name": "_setPendingOwner",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_newOracle",
        "type": "address"
      }
    ],
    "name": "_setPriceOracle",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setProtocolPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setRedeemPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_newRewardDistributor",
        "type": "address"
      }
    ],
    "name": "_setRewardDistributor",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setSeizePaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_newSupplyCapacity",
        "type": "uint256"
      }
    ],
    "name": "_setSupplyCapacity",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setTransferPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "_paused",
        "type": "bool"
      }
    ],
    "name": "_setiTokenPaused",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_borrowedAmount",
        "type": "uint256"
      }
    ],
    "name": "afterBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_amount",
        "type": "uint256"
      }
    ],
    "name": "afterFlashloan",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iTokenBorrowed",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iTokenCollateral",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_liquidator",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_repaidAmount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_seizedAmount",
        "type": "uint256"
      }
    ],
    "name": "afterLiquidateBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_minter",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_mintAmount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_mintedAmount",
        "type": "uint256"
      }
    ],
    "name": "afterMint",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_redeemer",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_redeemAmount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_redeemedUnderlying",
        "type": "uint256"
      }
    ],
    "name": "afterRedeem",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_payer",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_repayAmount",
        "type": "uint256"
      }
    ],
    "name": "afterRepayBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iTokenCollateral",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iTokenBorrowed",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_liquidator",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_seizedAmount",
        "type": "uint256"
      }
    ],
    "name": "afterSeize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_amount",
        "type": "uint256"
      }
    ],
    "name": "afterTransfer",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_borrowAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_amount",
        "type": "uint256"
      }
    ],
    "name": "beforeFlashloan",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iTokenBorrowed",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iTokenCollateral",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_liquidator",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_repayAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeLiquidateBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_minter",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_mintAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeMint",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_redeemer",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_redeemAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeRedeem",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_payer",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_repayAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeRepayBorrow",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iTokenCollateral",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iTokenBorrowed",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_liquidator",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_borrower",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_seizeAmount",
        "type": "uint256"
      }
    ],
    "name": "beforeSeize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_from",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_amount",
        "type": "uint256"
      }
    ],
    "name": "beforeTransfer",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_account",
        "type": "address"
      }
    ],
    "name": "calcAccountEquity",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "closeFactorMantissa",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address[]",
        "name": "_iTokens",
        "type": "address[]"
      }
    ],
    "name": "enterMarkets",
    "outputs": [
      {
        "internalType": "bool[]",
        "name": "_results",
        "type": "bool[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address[]",
        "name": "_iTokens",
        "type": "address[]"
      }
    ],
    "name": "exitMarkets",
    "outputs": [
      {
        "internalType": "bool[]",
        "name": "_results",
        "type": "bool[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getAlliTokens",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "_alliTokens",
        "type": "address[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_account",
        "type": "address"
      }
    ],
    "name": "getBorrowedAssets",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "_borrowedAssets",
        "type": "address[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_account",
        "type": "address"
      }
    ],
    "name": "getEnteredMarkets",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "_accountCollaterals",
        "type": "address[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_account",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      }
    ],
    "name": "hasBorrowed",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_account",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      }
    ],
    "name": "hasEnteredMarket",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iToken",
        "type": "address"
      }
    ],
    "name": "hasiToken",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "initialize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "isController",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_iTokenBorrowed",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_iTokenCollateral",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_actualRepayAmount",
        "type": "uint256"
      }
    ],
    "name": "liquidateCalculateSeizeTokens",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "_seizedTokenCollateral",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "liquidationIncentiveMantissa",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "name": "markets",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "collateralFactorMantissa",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "borrowFactorMantissa",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "borrowCapacity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "supplyCapacity",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "mintPaused",
        "type": "bool"
      },
      {
        "internalType": "bool",
        "name": "redeemPaused",
        "type": "bool"
      },
      {
        "internalType": "bool",
        "name": "borrowPaused",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "owner",
    "outputs": [
      {
        "internalType": "address payable",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "pauseGuardian",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "pendingOwner",
    "outputs": [
      {
        "internalType": "address payable",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "priceOracle",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "rewardDistributor",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "seizePaused",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "transferPaused",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
''')


import json

LIDO_LOCATOR_ABI = json.loads('''
[
  {
    "inputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "accountingOracle",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "depositSecurityModule",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "elRewardsVault",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "legacyOracle",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "lido",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "oracleReportSanityChecker",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "postTokenRebaseReceiver",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "burner",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "stakingRouter",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "treasury",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "validatorsExitBusOracle",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "withdrawalQueue",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "withdrawalVault",
            "type": "address"
          },
          {
            "internalType": "address",
            "name": "oracleDaemonConfig",
            "type": "address"
          }
        ],
        "internalType": "struct LidoLocator.Config",
        "name": "_config",
        "type": "tuple"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "inputs": [],
    "name": "ZeroAddress",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "accountingOracle",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "burner",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "coreComponents",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "depositSecurityModule",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "elRewardsVault",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "legacyOracle",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "lido",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "oracleDaemonConfig",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "oracleReportComponentsForLido",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "oracleReportSanityChecker",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "postTokenRebaseReceiver",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "stakingRouter",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "treasury",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "validatorsExitBusOracle",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "withdrawalQueue",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "withdrawalVault",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
''')


from defi_services.abis.dex.uniswap.factory_v2_abi import UNISWAP_FACTORY_ABI
from defi_services.abis.dex.uniswap.factory_v3_abi import UNISWAP_V3_FACTORY_ABI
from defi_services.abis.dex.uniswap.nft_token_manager_abi import UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI
from defi_services.abis.dex.uniswap.pool_v3_abi import UNISWAP_V3_POOL_ABI

UNISWAP_ETH_INFO = {
    'factory_address': '0x5c69bee701ef814a2b6a3edd4b1652cb9cc5aa6f',
    'factory_abi': UNISWAP_FACTORY_ABI,
    'forked': 'uniswap-v2'

}

UNISWAP_V3_ETH_INFO = {
    'factory_address': '0x1F98431c8aD98523631AE4a59f267346ea31F984',
    'factory_abi': UNISWAP_V3_FACTORY_ABI,
    'staker_address': '0xe34139463bA50bD61336E0c446Bd8C0867c6fE65',
    'NFT_manager_address': '0xC36442b4a4522E871399CD717aBDD847Ab11FE88',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}




import json

AAVE_ORACLE_V3 = json.loads("""
[
    {
      "inputs": [
        {
          "internalType": "contract IPoolAddressesProvider",
          "name": "provider",
          "type": "address"
        },
        {
          "internalType": "address[]",
          "name": "assets",
          "type": "address[]"
        },
        {
          "internalType": "address[]",
          "name": "sources",
          "type": "address[]"
        },
        {
          "internalType": "address",
          "name": "fallbackOracle",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "baseCurrency",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "baseCurrencyUnit",
          "type": "uint256"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "asset",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "source",
          "type": "address"
        }
      ],
      "name": "AssetSourceUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "baseCurrency",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "baseCurrencyUnit",
          "type": "uint256"
        }
      ],
      "name": "BaseCurrencySet",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "fallbackOracle",
          "type": "address"
        }
      ],
      "name": "FallbackOracleUpdated",
      "type": "event"
    },
    {
      "inputs": [],
      "name": "ADDRESSES_PROVIDER",
      "outputs": [
        {
          "internalType": "contract IPoolAddressesProvider",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "BASE_CURRENCY",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "BASE_CURRENCY_UNIT",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "asset",
          "type": "address"
        }
      ],
      "name": "getAssetPrice",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address[]",
          "name": "assets",
          "type": "address[]"
        }
      ],
      "name": "getAssetsPrices",
      "outputs": [
        {
          "internalType": "uint256[]",
          "name": "",
          "type": "uint256[]"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getFallbackOracle",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "asset",
          "type": "address"
        }
      ],
      "name": "getSourceOfAsset",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address[]",
          "name": "assets",
          "type": "address[]"
        },
        {
          "internalType": "address[]",
          "name": "sources",
          "type": "address[]"
        }
      ],
      "name": "setAssetSources",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "fallbackOracle",
          "type": "address"
        }
      ],
      "name": "setFallbackOracle",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    }
]
""")
        


from defi_services.services.dex.pancakeswap_v2_service import PancakeSwapV2Services
from defi_services.services.dex.pancakeswap_v3_service import PancakeSwapV3Service
from defi_services.services.dex.quickswap_v2_service import QuickSwapV2Services
from defi_services.services.dex.quickswap_v3_service import QuickSwapV3Services
from defi_services.services.dex.spookyswap_v2_service import SpookySwapV2Services
from defi_services.services.dex.sushiswap_v2_service import SushiSwapV2Services
from defi_services.services.dex.sushiswap_v3_service import SushiSwapV3Services
from defi_services.services.dex.uniswap_v2_service import UniswapV2Services
from defi_services.services.dex.uniswap_v3_service import UniswapV3Services

from defi_services.constants.chain_constant import Chain
from defi_services.constants.entities.dex_constant import Dex
from defi_services.services.dex.pancakeswap_service import PancakeSwapServices
from defi_services.services.dex.sushiswap_service import SushiSwapServices


class DexServices:
    ethereum = {
        Dex.uniswap_v2: UniswapV2Services,
        Dex.uniswap_v3: UniswapV3Services,
        Dex.sushi: SushiSwapServices,
        Dex.sushi_v2: SushiSwapV2Services,
        Dex.sushi_v3: SushiSwapV3Services,

    }
    bsc = {
        Dex.pancake: PancakeSwapServices,
        Dex.pancake_v3: PancakeSwapV3Service,
        Dex.pancake_v2: PancakeSwapV2Services,
        Dex.sushi_v2: SushiSwapV2Services
    }
    avalanche = {
        Dex.sushi_v2: SushiSwapV2Services
    }
    polygon = {
        Dex.sushi_v2: SushiSwapV2Services,
        Dex.quickswap_v2: QuickSwapV2Services,
        Dex.quickswap_v3: QuickSwapV3Services
    }
    fantom = {
        Dex.sushi_v2: SushiSwapV2Services,
        Dex.spooky_v2: SpookySwapV2Services
    }
    # optimism = {
    #     Dex.sushi: SushiSwapV2Services
    # }
    arbitrum = {
        Dex.sushi_v2: SushiSwapV2Services
    }
    mapping = {
        Chain.ethereum: ethereum,
        Chain.fantom: fantom,
        Chain.bsc: bsc,
        Chain.avalanche: avalanche,
        Chain.polygon: polygon,
        Chain.arbitrum: arbitrum,
        # Chain.optimism: optimism,
    }




import json

ORACLE_ANCHOR = json.loads("""
[
    {
      "inputs": [
        {
          "internalType": "address[]",
          "name": "assets",
          "type": "address[]"
        },
        {
          "internalType": "address[]",
          "name": "sources",
          "type": "address[]"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "token",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "source",
          "type": "address"
        }
      ],
      "name": "AssetSourceUpdated",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [],
      "name": "OracleSystemMigrated",
      "type": "event"
    }
]
""")
        


SUSHISWAP_V3_ETH_FACTORY_ABI = [{"inputs":[],"stateMutability":"nonpayable","type":"constructor"},{"anonymous":False,"inputs":[{"indexed":True,"internalType":"uint24","name":"fee","type":"uint24"},{"indexed":True,"internalType":"int24","name":"tickSpacing","type":"int24"}],"name":"FeeAmountEnabled","type":"event"},{"anonymous":False,"inputs":[{"indexed":True,"internalType":"address","name":"oldOwner","type":"address"},{"indexed":True,"internalType":"address","name":"newOwner","type":"address"}],"name":"OwnerChanged","type":"event"},{"anonymous":False,"inputs":[{"indexed":True,"internalType":"address","name":"token0","type":"address"},{"indexed":True,"internalType":"address","name":"token1","type":"address"},{"indexed":True,"internalType":"uint24","name":"fee","type":"uint24"},{"indexed":False,"internalType":"int24","name":"tickSpacing","type":"int24"},{"indexed":False,"internalType":"address","name":"pool","type":"address"}],"name":"PoolCreated","type":"event"},{"inputs":[{"internalType":"address","name":"tokenA","type":"address"},{"internalType":"address","name":"tokenB","type":"address"},{"internalType":"uint24","name":"fee","type":"uint24"}],"name":"createPool","outputs":[{"internalType":"address","name":"pool","type":"address"}],"stateMutability":"nonpayable","type":"function"},{"inputs":[{"internalType":"uint24","name":"fee","type":"uint24"},{"internalType":"int24","name":"tickSpacing","type":"int24"}],"name":"enableFeeAmount","outputs":[],"stateMutability":"nonpayable","type":"function"},{"inputs":[{"internalType":"uint24","name":"","type":"uint24"}],"name":"feeAmountTickSpacing","outputs":[{"internalType":"int24","name":"","type":"int24"}],"stateMutability":"view","type":"function"},{"inputs":[{"internalType":"address","name":"","type":"address"},{"internalType":"address","name":"","type":"address"},{"internalType":"uint24","name":"","type":"uint24"}],"name":"getPool","outputs":[{"internalType":"address","name":"","type":"address"}],"stateMutability":"view","type":"function"},{"inputs":[],"name":"owner","outputs":[{"internalType":"address","name":"","type":"address"}],"stateMutability":"view","type":"function"},{"inputs":[],"name":"parameters","outputs":[{"internalType":"address","name":"factory","type":"address"},{"internalType":"address","name":"token0","type":"address"},{"internalType":"address","name":"token1","type":"address"},{"internalType":"uint24","name":"fee","type":"uint24"},{"internalType":"int24","name":"tickSpacing","type":"int24"}],"stateMutability":"view","type":"function"},{"inputs":[{"internalType":"address","name":"_owner","type":"address"}],"name":"setOwner","outputs":[],"stateMutability":"nonpayable","type":"function"}]


PANCAKESWAP_MASTERCHEF_V3_ABI = [{
                                     "inputs": [{"internalType": "contract IERC20", "name": "_CAKE", "type": "address"},
                                                {
                                                    "internalType": "contract INonfungiblePositionManager",
                                                    "name": "_nonfungiblePositionManager", "type": "address"},
                                                {"internalType": "address", "name": "_WETH", "type": "address"}],
                                     "stateMutability": "nonpayable", "type": "constructor"}, {
                                     "inputs": [{"internalType": "uint256", "name": "pid", "type": "uint256"}],
                                     "name": "DuplicatedPool", "type": "error"},
                                 {"inputs": [], "name": "InconsistentAmount", "type": "error"},
                                 {"inputs": [], "name": "InsufficientAmount", "type": "error"},
                                 {"inputs": [], "name": "InvalidNFT", "type": "error"},
                                 {"inputs": [], "name": "InvalidPeriodDuration", "type": "error"},
                                 {"inputs": [], "name": "InvalidPid", "type": "error"},
                                 {"inputs": [], "name": "NoBalance", "type": "error"},
                                 {"inputs": [], "name": "NoLMPool", "type": "error"},
                                 {"inputs": [], "name": "NoLiquidity", "type": "error"},
                                 {"inputs": [], "name": "NotEmpty", "type": "error"},
                                 {"inputs": [], "name": "NotOwner", "type": "error"},
                                 {"inputs": [], "name": "NotOwnerOrOperator", "type": "error"},
                                 {"inputs": [], "name": "NotPancakeNFT", "type": "error"},
                                 {"inputs": [], "name": "WrongReceiver", "type": "error"},
                                 {"inputs": [], "name": "ZeroAddress", "type": "error"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "allocPoint", "type": "uint256"},
            {"indexed": True, "internalType": "contract IPancakeV3Pool", "name": "v3Pool", "type": "address"},
            {"indexed": True, "internalType": "contract ILMPool", "name": "lmPool", "type": "address"}],
                                     "name": "AddPool", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "from", "type": "address"},
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": True, "internalType": "uint256", "name": "tokenId", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "liquidity", "type": "uint256"},
            {"indexed": False, "internalType": "int24", "name": "tickLower", "type": "int24"},
            {"indexed": False, "internalType": "int24", "name": "tickUpper", "type": "int24"}], "name": "Deposit",
                                     "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "sender", "type": "address"},
            {"indexed": False, "internalType": "address", "name": "to", "type": "address"},
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": True, "internalType": "uint256", "name": "tokenId", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "reward", "type": "uint256"}], "name": "Harvest",
                                     "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": False, "internalType": "address", "name": "deployer", "type": "address"}],
                                     "name": "NewLMPoolDeployerAddress", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": False, "internalType": "address", "name": "operator", "type": "address"}],
                                     "name": "NewOperatorAddress", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": False, "internalType": "uint256", "name": "periodDuration", "type": "uint256"}],
                                     "name": "NewPeriodDuration", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": False, "internalType": "address", "name": "receiver", "type": "address"}],
                                     "name": "NewReceiver", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "uint256", "name": "periodNumber", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "startTime", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "endTime", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "cakePerSecond", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "cakeAmount", "type": "uint256"}],
                                     "name": "NewUpkeepPeriod", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "previousOwner", "type": "address"},
            {"indexed": True, "internalType": "address", "name": "newOwner", "type": "address"}],
                                     "name": "OwnershipTransferred", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": False, "internalType": "bool", "name": "emergency", "type": "bool"}], "name": "SetEmergency",
                                     "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "allocPoint", "type": "uint256"}], "name": "SetPool",
                                     "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "farmBoostContract", "type": "address"}],
                                     "name": "UpdateFarmBoostContract", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "from", "type": "address"},
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": True, "internalType": "uint256", "name": "tokenId", "type": "uint256"},
            {"indexed": False, "internalType": "int128", "name": "liquidity", "type": "int128"},
            {"indexed": False, "internalType": "int24", "name": "tickLower", "type": "int24"},
            {"indexed": False, "internalType": "int24", "name": "tickUpper", "type": "int24"}],
                                     "name": "UpdateLiquidity", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "uint256", "name": "periodNumber", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "oldEndTime", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "newEndTime", "type": "uint256"},
            {"indexed": False, "internalType": "uint256", "name": "remainingCake", "type": "uint256"}],
                                     "name": "UpdateUpkeepPeriod", "type": "event"}, {
                                     "anonymous": False, "inputs": [
            {"indexed": True, "internalType": "address", "name": "from", "type": "address"},
            {"indexed": False, "internalType": "address", "name": "to", "type": "address"},
            {"indexed": True, "internalType": "uint256", "name": "pid", "type": "uint256"},
            {"indexed": True, "internalType": "uint256", "name": "tokenId", "type": "uint256"}], "name": "Withdraw",
                                     "type": "event"}, {
                                     "inputs": [], "name": "BOOST_PRECISION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "CAKE",
                                     "outputs": [{"internalType": "contract IERC20", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "FARM_BOOSTER", "outputs": [
            {"internalType": "contract IFarmBooster", "name": "", "type": "address"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [], "name": "LMPoolDeployer", "outputs": [
            {"internalType": "contract ILMPoolDeployer", "name": "", "type": "address"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [], "name": "MAX_BOOST_PRECISION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "MAX_DURATION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "MIN_DURATION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "PERIOD_DURATION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "PRECISION",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "WETH",
                                     "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_allocPoint", "type": "uint256"}, {
                                         "internalType": "contract IPancakeV3Pool", "name": "_v3Pool",
                                         "type": "address"},
                                                {"internalType": "bool", "name": "_withUpdate", "type": "bool"}],
                                     "name": "add", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
                                 {
                                     "inputs": [{"internalType": "address", "name": "owner", "type": "address"}],
                                     "name": "balanceOf",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"}],
                                     "name": "burn", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [], "name": "cakeAmountBelongToMC",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{
                                                    "components": [{
                                                                       "internalType": "uint256", "name": "tokenId",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "address", "name": "recipient",
                                                                       "type": "address"}, {
                                                                       "internalType": "uint128", "name": "amount0Max",
                                                                       "type": "uint128"}, {
                                                                       "internalType": "uint128", "name": "amount1Max",
                                                                       "type": "uint128"}],
                                                    "internalType": "struct INonfungiblePositionManagerStruct.CollectParams",
                                                    "name": "params", "type": "tuple"}], "name": "collect",
                                     "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                                                 {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [{
                                                    "components": [{
                                                                       "internalType": "uint256", "name": "tokenId",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "address", "name": "recipient",
                                                                       "type": "address"}, {
                                                                       "internalType": "uint128", "name": "amount0Max",
                                                                       "type": "uint128"}, {
                                                                       "internalType": "uint128", "name": "amount1Max",
                                                                       "type": "uint128"}],
                                                    "internalType": "struct INonfungiblePositionManagerStruct.CollectParams",
                                                    "name": "params", "type": "tuple"},
                                                {"internalType": "address", "name": "to", "type": "address"}],
                                     "name": "collectTo",
                                     "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                                                 {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [{
                                                    "components": [{
                                                                       "internalType": "uint256", "name": "tokenId",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint128", "name": "liquidity",
                                                                       "type": "uint128"}, {
                                                                       "internalType": "uint256", "name": "amount0Min",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint256", "name": "amount1Min",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint256", "name": "deadline",
                                                                       "type": "uint256"}],
                                                    "internalType": "struct INonfungiblePositionManagerStruct.DecreaseLiquidityParams",
                                                    "name": "params", "type": "tuple"}], "name": "decreaseLiquidity",
                                     "outputs": [{"internalType": "uint256", "name": "amount0", "type": "uint256"},
                                                 {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [], "name": "emergency",
                                     "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "_v3Pool", "type": "address"}],
                                     "name": "getLatestPeriodInfo", "outputs": [
            {"internalType": "uint256", "name": "cakePerSecond", "type": "uint256"},
            {"internalType": "uint256", "name": "endTime", "type": "uint256"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"}],
                                     "name": "getLatestPeriodInfoByPid", "outputs": [
            {"internalType": "uint256", "name": "cakePerSecond", "type": "uint256"},
            {"internalType": "uint256", "name": "endTime", "type": "uint256"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"},
                                                {"internalType": "address", "name": "_to", "type": "address"}],
                                     "name": "harvest",
                                     "outputs": [{"internalType": "uint256", "name": "reward", "type": "uint256"}],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [{
                                                    "components": [{
                                                                       "internalType": "uint256", "name": "tokenId",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint256",
                                                                       "name": "amount0Desired", "type": "uint256"}, {
                                                                       "internalType": "uint256",
                                                                       "name": "amount1Desired", "type": "uint256"}, {
                                                                       "internalType": "uint256", "name": "amount0Min",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint256", "name": "amount1Min",
                                                                       "type": "uint256"}, {
                                                                       "internalType": "uint256", "name": "deadline",
                                                                       "type": "uint256"}],
                                                    "internalType": "struct INonfungiblePositionManagerStruct.IncreaseLiquidityParams",
                                                    "name": "params", "type": "tuple"}], "name": "increaseLiquidity",
                                     "outputs": [{"internalType": "uint128", "name": "liquidity", "type": "uint128"},
                                                 {"internalType": "uint256", "name": "amount0", "type": "uint256"},
                                                 {"internalType": "uint256", "name": "amount1", "type": "uint256"}],
                                     "stateMutability": "payable", "type": "function"}, {
                                     "inputs": [], "name": "latestPeriodCakePerSecond",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "latestPeriodEndTime",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "latestPeriodNumber",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "latestPeriodStartTime",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "bytes[]", "name": "data", "type": "bytes[]"}],
                                     "name": "multicall",
                                     "outputs": [{"internalType": "bytes[]", "name": "results", "type": "bytes[]"}],
                                     "stateMutability": "payable", "type": "function"}, {
                                     "inputs": [], "name": "nonfungiblePositionManager", "outputs": [
            {"internalType": "contract INonfungiblePositionManager", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "", "type": "address"},
                                                {"internalType": "address", "name": "_from", "type": "address"},
                                                {"internalType": "uint256", "name": "_tokenId", "type": "uint256"},
                                                {"internalType": "bytes", "name": "", "type": "bytes"}],
                                     "name": "onERC721Received",
                                     "outputs": [{"internalType": "bytes4", "name": "", "type": "bytes4"}],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [], "name": "operatorAddress",
                                     "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "owner",
                                     "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"}],
                                     "name": "pendingCake",
                                     "outputs": [{"internalType": "uint256", "name": "reward", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "name": "poolInfo",
                                     "outputs": [{"internalType": "uint256", "name": "allocPoint", "type": "uint256"}, {
                                         "internalType": "contract IPancakeV3Pool", "name": "v3Pool",
                                         "type": "address"},
                                                 {"internalType": "address", "name": "token0", "type": "address"},
                                                 {"internalType": "address", "name": "token1", "type": "address"},
                                                 {"internalType": "uint24", "name": "fee", "type": "uint24"}, {
                                                     "internalType": "uint256", "name": "totalLiquidity",
                                                     "type": "uint256"}, {
                                                     "internalType": "uint256", "name": "totalBoostLiquidity",
                                                     "type": "uint256"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [], "name": "poolLength",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "receiver",
                                     "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "renounceOwnership", "outputs": [],
                                     "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_pid", "type": "uint256"},
                                                {"internalType": "uint256", "name": "_allocPoint", "type": "uint256"},
                                                {"internalType": "bool", "name": "_withUpdate", "type": "bool"}],
                                     "name": "set", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
                                 {
                                     "inputs": [{"internalType": "bool", "name": "_emergency", "type": "bool"}],
                                     "name": "setEmergency", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{
                                                    "internalType": "contract ILMPoolDeployer",
                                                    "name": "_LMPoolDeployer", "type": "address"}],
                                     "name": "setLMPoolDeployer", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [
                                         {"internalType": "address", "name": "_operatorAddress", "type": "address"}],
                                     "name": "setOperator", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [
                                         {"internalType": "uint256", "name": "_periodDuration", "type": "uint256"}],
                                     "name": "setPeriodDuration", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "_receiver", "type": "address"}],
                                     "name": "setReceiver", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "token", "type": "address"},
                                                {"internalType": "uint256", "name": "amountMinimum", "type": "uint256"},
                                                {"internalType": "address", "name": "recipient", "type": "address"}],
                                     "name": "sweepToken", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "owner", "type": "address"},
                                                {"internalType": "uint256", "name": "index", "type": "uint256"}],
                                     "name": "tokenOfOwnerByIndex",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [], "name": "totalAllocPoint",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "newOwner", "type": "address"}],
                                     "name": "transferOwnership", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "amountMinimum", "type": "uint256"},
                                                {"internalType": "address", "name": "recipient", "type": "address"}],
                                     "name": "unwrapWETH9", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"}, {
                                         "internalType": "uint256", "name": "_newMultiplier", "type": "uint256"}],
                                     "name": "updateBoostMultiplier", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{
                                                    "internalType": "address", "name": "_newFarmBoostContract",
                                                    "type": "address"}], "name": "updateFarmBoostContract",
                                     "outputs": [], "stateMutability": "nonpayable", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"}],
                                     "name": "updateLiquidity", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256[]", "name": "pids", "type": "uint256[]"}],
                                     "name": "updatePools", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_amount", "type": "uint256"},
                                                {"internalType": "uint256", "name": "_duration", "type": "uint256"},
                                                {"internalType": "bool", "name": "_withUpdate", "type": "bool"}],
                                     "name": "upkeep", "outputs": [], "stateMutability": "nonpayable",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "name": "userPositionInfos",
                                     "outputs": [{"internalType": "uint128", "name": "liquidity", "type": "uint128"}, {
                                         "internalType": "uint128", "name": "boostLiquidity", "type": "uint128"},
                                                 {"internalType": "int24", "name": "tickLower", "type": "int24"},
                                                 {"internalType": "int24", "name": "tickUpper", "type": "int24"}, {
                                                     "internalType": "uint256", "name": "rewardGrowthInside",
                                                     "type": "uint256"},
                                                 {"internalType": "uint256", "name": "reward", "type": "uint256"},
                                                 {"internalType": "address", "name": "user", "type": "address"},
                                                 {"internalType": "uint256", "name": "pid", "type": "uint256"}, {
                                                     "internalType": "uint256", "name": "boostMultiplier",
                                                     "type": "uint256"}], "stateMutability": "view",
                                     "type": "function"}, {
                                     "inputs": [{"internalType": "address", "name": "", "type": "address"}],
                                     "name": "v3PoolAddressPid",
                                     "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                     "stateMutability": "view", "type": "function"}, {
                                     "inputs": [{"internalType": "uint256", "name": "_tokenId", "type": "uint256"},
                                                {"internalType": "address", "name": "_to", "type": "address"}],
                                     "name": "withdraw",
                                     "outputs": [{"internalType": "uint256", "name": "reward", "type": "uint256"}],
                                     "stateMutability": "nonpayable", "type": "function"},
                                 {"stateMutability": "payable", "type": "receive"}]
